#include "mainwindow.h"
#include <cctype>
#include <cmath>
#define SEPARATOR '.'

Calculator::Calculator(QWidget *parent) : QWidget(parent)
{

    if (objectName().isEmpty())
    {
        setObjectName(QString::fromUtf8("MainWindow"));
    }

    resize(16, 16);
    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);

    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(this->sizePolicy().hasHeightForWidth());
    setSizePolicy(sizePolicy);

    mSimpleModeRadioButton = new QRadioButton(this);
    mSimpleModeRadioButton->setObjectName(QString::fromUtf8("SimpleRadioButton"));
    mSimpleModeRadioButton->setGeometry(QRect(0, 0, 250, 10));

    mEngineeringModeRadioButton = new QRadioButton(this);
    mEngineeringModeRadioButton->setObjectName(QString::fromUtf8("EngRadioButton"));
    mEngineeringModeRadioButton->setGeometry(QRect(0, 0, 150, 10));

    mLineEdit = new QLineEdit(this);
    mLineEdit->setObjectName(QString::fromUtf8("LineEdit"));
    mLineEdit->setGeometry(QRect(0, 0, 108, 22));
    mLineEdit->setPlaceholderText("0");
    mLineEdit->setReadOnly(true);

    mSimpleModeRadioButton->setText(tr("Обычный"));
    mEngineeringModeRadioButton->setText(tr("Инженерный"));

    setWindowTitle(tr("Калькулятор(обычный)"));
    setWindowOpacity(1);
    setMinimumSize(350, 420);
    setMaximumSize(350, 420);

    mSimpleModeRadioButton->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    mEngineeringModeRadioButton->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred);
    mSimpleModeRadioButton->setChecked(true);

    connect(mSimpleModeRadioButton, &QRadioButton::toggled, this, &Calculator::SwitchMode);

    createSimpleCalcWidget();
    createEngineeringCalcWidget();
    createDefaultCalcWidget();
    createSimpleCalc();
}

void Calculator::createSimpleCalcWidget()
{

    QPushButton* button0         = new QPushButton("0");
    QPushButton* button1         = new QPushButton("1");
    QPushButton* button2         = new QPushButton("2");
    QPushButton* button3         = new QPushButton("3");
    QPushButton* button4         = new QPushButton("4");
    QPushButton* button5         = new QPushButton("5");
    QPushButton* button6         = new QPushButton("6");
    QPushButton* button7         = new QPushButton("7");
    QPushButton* button8         = new QPushButton("8");
    QPushButton* button9         = new QPushButton("9");
    QPushButton* buttonDot       = new QPushButton(".");
    QPushButton* buttonPlus      = new QPushButton("+");
    QPushButton* buttonEqual     = new QPushButton("=");
    QPushButton* buttonReverse   = new QPushButton("1/x");

    QPushButton* buttonSqrt      = new QPushButton(QString::fromUtf8("\u221Ax"));
    QPushButton* buttonMinus     = new QPushButton(QString::fromUtf8("-"));
    QPushButton* buttonMult      = new QPushButton(QString::fromUtf8("\u00D7"));
    QPushButton* buttonDivide    = new QPushButton(QString::fromUtf8("\u00F7"));
    QPushButton* buttonSign      = new QPushButton(QString::fromUtf8("\u00B1"));

    button0->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button1->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button2->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button3->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button4->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button5->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button6->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button7->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button8->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    button9->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);

    buttonPlus   ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    buttonMult   ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    buttonDivide ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    buttonEqual  ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    buttonReverse->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    buttonSqrt   ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    buttonSign   ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    buttonMinus  ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    buttonDot    ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);

    mSimpleCalculatorLayout = new QGridLayout(this);
    mSimpleCalculatorLayout->setSpacing(0);
    mSimpleCalculatorLayout->setContentsMargins(0, 0, 0, 0);

    mSimpleCalculatorLayout->addWidget(button7      , 0, 0, 1, 1);
    mSimpleCalculatorLayout->addWidget(button8      , 0, 1, 1, 1);
    mSimpleCalculatorLayout->addWidget(button9      , 0, 2, 1, 1);
    mSimpleCalculatorLayout->addWidget(buttonDivide , 0, 3, 1, 1);
    mSimpleCalculatorLayout->addWidget(buttonSqrt   , 0, 4, 1, 1);
    mSimpleCalculatorLayout->addWidget(button4      , 1, 0, 1, 1);
    mSimpleCalculatorLayout->addWidget(button5      , 1, 1, 1, 1);
    mSimpleCalculatorLayout->addWidget(button6      , 1, 2, 1, 1);
    mSimpleCalculatorLayout->addWidget(buttonMult   , 1, 3, 1, 1);
    mSimpleCalculatorLayout->addWidget(buttonReverse, 1, 4, 1, 1);
    mSimpleCalculatorLayout->addWidget(button1      , 2, 0, 1, 1);
    mSimpleCalculatorLayout->addWidget(button2      , 2, 1, 1, 1);
    mSimpleCalculatorLayout->addWidget(button3      , 2, 2, 1, 1);
    mSimpleCalculatorLayout->addWidget(buttonMinus  , 2, 3, 1, 1);
    mSimpleCalculatorLayout->addWidget(buttonEqual  , 3, 4, 1, 1);
    mSimpleCalculatorLayout->addWidget(button0      , 3, 0, 1, 2);
    mSimpleCalculatorLayout->addWidget(buttonDot    , 3, 2, 1, 1);
    mSimpleCalculatorLayout->addWidget(buttonPlus   , 3, 3, 1, 1);
    mSimpleCalculatorLayout->addWidget(buttonSign   , 2, 4, 1, 1);

    mSimpleCalculatorWidget = new QWidget(this);
    mSimpleCalculatorWidget->setLayout(mSimpleCalculatorLayout);

    connect(button0      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button1      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button2      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button3      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button4      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button5      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button6      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button7      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button8      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(button9      , &QPushButton::clicked, this, &Calculator::NumberClicked);
    connect(buttonPlus   , &QPushButton::clicked, this, &Calculator::BinaryOpClicked);
    connect(buttonMinus  , &QPushButton::clicked, this, &Calculator::BinaryOpClicked);
    connect(buttonDot    , &QPushButton::clicked, this, &Calculator::DotClicked);
    connect(buttonEqual  , &QPushButton::clicked, this, &Calculator::Equal);
    connect(buttonReverse, &QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(buttonMult   , &QPushButton::clicked, this, &Calculator::BinaryOpClicked);
    connect(buttonDivide , &QPushButton::clicked, this, &Calculator::BinaryOpClicked);
    connect(buttonSqrt   , &QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(buttonSign   , &QPushButton::clicked, this, &Calculator::UnaryOpClicked);
}

void Calculator::createEngineeringCalcWidget()
{
    QPushButton* pushButtonSinh     = new QPushButton("sinh");
    QPushButton* pushButtonSin      = new QPushButton("sin");
    QPushButton* pushButtonExp      = new QPushButton(QString::fromUtf8("e\u207F"));
    QPushButton* pushButtonXSq      = new QPushButton(QString::fromUtf8("x\u00B2"));
    QPushButton* pushButtonCosh     = new QPushButton("cosh");
    QPushButton* pushButtonCos      = new QPushButton("cos");
    QPushButton* pushButtonLn       = new QPushButton("ln");
    QPushButton* pushButtonXCubed   = new QPushButton(QString::fromUtf8("x\u00B3"));
    QPushButton* pushButtonTanh     = new QPushButton("tanh");
    QPushButton* pushButtonTan      = new QPushButton("tan");
    QPushButton* pushButtonLog      = new QPushButton("log");
    QPushButton* pushButtonXNed     = new QPushButton(QString::fromUtf8("x\u207F"));
    QPushButton* pushButtonFact     = new QPushButton("n!");
    QPushButton* pushButtonPi       = new QPushButton(QString::fromUtf8("\u03C0"));
    QPushButton* pushButtonCubeRoot = new QPushButton(QString::fromUtf8("\u00B3\u221A"));
    QPushButton* pushButtonNRoot    = new QPushButton(QString::fromUtf8("\u207F\u221A"));

    pushButtonSinh    ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonSin     ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonExp     ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonXSq     ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonCosh    ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonCos     ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonLn      ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonXCubed  ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonTanh    ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonTan     ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonLog     ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonXNed    ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonFact    ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonPi      ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonCubeRoot->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    pushButtonNRoot   ->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);

    mEngineeringCalcLayout = new QGridLayout(this);
    mEngineeringCalcLayout->setSpacing(0);
    mEngineeringCalcLayout->setContentsMargins(0, 0, 0, 0);

    mEngineeringCalcLayout->addWidget(pushButtonSinh    , 0, 0, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonSin     , 0, 1, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonExp     , 0, 2, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonXSq     , 0, 3, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonCosh    , 1, 0, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonCos     , 1, 1, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonLn      , 1, 2, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonXCubed  , 1, 3, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonTanh    , 2, 0, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonTan     , 2, 1, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonLog     , 2, 2, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonXNed    , 2, 3, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonFact    , 3, 0, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonPi      , 3, 1, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonCubeRoot, 3, 2, 1, 1);
    mEngineeringCalcLayout->addWidget(pushButtonNRoot   , 3, 3, 1, 1);


    mEngineeringCalcWidget = new QWidget(this);
    mEngineeringCalcWidget->setLayout(mEngineeringCalcLayout);

    connect(pushButtonSin      ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonSinh     ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonCos      ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonCosh     ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonFact     ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonTan      ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonTanh     ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonLn       ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonLog      ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonXNed     ,&QPushButton::clicked, this, &Calculator::BinaryOpClicked);
    connect(pushButtonXSq      ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonXCubed   ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonCubeRoot ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonNRoot    ,&QPushButton::clicked, this, &Calculator::BinaryOpClicked);
    connect(pushButtonExp      ,&QPushButton::clicked, this, &Calculator::UnaryOpClicked);
    connect(pushButtonPi       ,&QPushButton::clicked, this, &Calculator::NumberClicked);
}

void Calculator::createDefaultCalcWidget()
{
    QPushButton* pushButtonClear =  new QPushButton(QString::fromUtf8("C"), this);

    QString StyleSheetLine = "QLineEdit {font: 25pt 'Microsoft YaHei UI';"
                             " qproperty-alignment: AlignRight;"
                             " padding: 5px;"
                             " border: none; background-color: #F2F2F2;}";

    mLineEdit->setStyleSheet(StyleSheetLine);

    pushButtonClear->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);

    mDefaultCalculatorLayout = new QGridLayout(this);
    mDefaultCalculatorLayout->setSpacing(0);
    mDefaultCalculatorLayout->setContentsMargins(0, 0, 0, 0);

    mDefaultCalculatorLayout->addWidget(mLineEdit                  , 0, 0, 2, 5);
    mDefaultCalculatorLayout->addWidget(mSimpleModeRadioButton     , 2, 0, 1, 5);
    mDefaultCalculatorLayout->addWidget(mEngineeringModeRadioButton, 3, 0, 1, 5);
    mDefaultCalculatorLayout->addWidget(pushButtonClear            , 2, 4, 2, 1);

    mDefaultCalculatorWidget = new QWidget(this);
    mDefaultCalculatorWidget->setLayout(mDefaultCalculatorLayout);

    connect(pushButtonClear, &QPushButton::clicked, this, &Calculator::Clear);
}

void Calculator::SwitchMode()
{
    mSimpleCalculatorWidget->setParent(this);
    mEngineeringCalcWidget->setParent(this);
    if (!mSimpleModeRadioButton->isChecked())
    {
        delete mMainLayout;

        setWindowTitle(tr("Калькулятор (инженерный)"));
        setMinimumSize(700, 500);
        setMaximumSize(700, 500);
        createEngineeringCalc();
    }
    else
    {
        delete mMainLayout;

        setWindowTitle(tr("Калькулятор (обычный)"));
        setMinimumSize(400, 500);
        setMaximumSize(400, 500);
        createSimpleCalc();
    }
}

void Calculator::createSimpleCalc()
{
    mMainLayout = new QGridLayout;
    mMainLayout->setSpacing(0);
    mEngineeringCalcWidget->hide();
    mMainLayout->addWidget(mDefaultCalculatorWidget, 0, 0, 1, 3);
    mMainLayout->addWidget(mSimpleCalculatorWidget,  1, 0, 2, 3);

    setLayout(mMainLayout);
    layout()->setContentsMargins(0, 0, 0, 0);
}

void Calculator::createEngineeringCalc()
{
    mMainLayout = new QGridLayout;
    mMainLayout->setSpacing(0);
    mMainLayout->addWidget(mDefaultCalculatorWidget, 0, 0, 1, 9);
    mMainLayout->addWidget(mEngineeringCalcWidget,   1, 0, 2, 4);
    mMainLayout->addWidget(mSimpleCalculatorWidget,  1, 4, 2, 5);
    mEngineeringCalcWidget->show();

    setLayout(mMainLayout);
    layout()->setContentsMargins(0, 0, 0, 0);
}

void Calculator::UnaryOpClicked()
{
    if (mLineEdit->text().toStdString() == "")
        return;

    Equal();
    SetDigits(true);
    std::string symbol = (qobject_cast<QPushButton*>(sender())->text().toStdString());
    std::string expression = mLineEdit->text().toStdString();
    double number = std::stod(expression);

    if (symbol == "\u221Ax")
    {
        number = std::sqrt(number);
    }
    else if (symbol == "sin")
    {
        number = std::sin(number);
    }
    else if (symbol == "sinh")
    {
        number = std::sinh(number);
    }
    else if (symbol == "e\u207F")
    {
        number = std::exp(number);
    }
    else if (symbol == "x\u00B2")
    {
        number *= number;
    }
    else if (symbol == "cosh")
    {
        number = std::cosh(number);
    }
    else if (symbol == "cos")
    {
        number = std::cos(number);
    }
    else if (symbol == "ln")
    {
        number = std::log(number);
    }
    else if (symbol == "\u00B3\u221A")
    {
        number = std::cbrt(number);
    }
    else if (symbol == "x\u00B3")
    {
        number *= (number * number);
    }
    else if (symbol == "tanh")
    {
        number = std::tanh(number);
    }
    else if (symbol == "tan")
    {
        number = std::tan(number);
    }
    else if (symbol == "\u00B1")
    {
        number = -number;
    }
    else if (symbol == "log")
    {
        number = std::log10(number);
    }
    else if (symbol == "n!")
    {
        if (std::floor(number) != number)
        {
            mLineEdit->setText(QString("nan"));
            return;
        }
        else
        {
            number = fact(number);
        }
    }
    else if (symbol == "1/x")
    {
        number = 1 / number;
        expression = std::to_string(number);
        mLineEdit->setText(QString(expression.c_str()));
    }

    expression = std::to_string(number);
    while (*(expression.rbegin()) == '0')
    {
        expression.pop_back();
    }
    if (*(expression.rbegin()) == SEPARATOR)
    {
        expression.pop_back();
    }
    mLineEdit->setText(QString(expression.c_str()));
}

bool Calculator::OneDot(bool p)
{
    mDotOne = p;
}

void Calculator::NumberClicked()
{
    QString number = (qobject_cast<QPushButton*>(sender()))->text();
    if (number == "\u03C0")
    {
        constexpr double pi = 3.1415926535897932;
        mLineEdit->setText(mLineEdit->text() + QString::number(pi, 'f'));
    }
    else
    {
        mLineEdit->setText(mLineEdit->text() + number);
    }
}

void Calculator::SetDigits(bool new_state)
{
    mDigitsOonly = new_state;
}

void Calculator::DotClicked()
{
    if(mDotOne==true)
    {
        return;
    }
    OneDot(true);
    if (mLineEdit->text().length())
    {
        if (mLineEdit->text().at(mLineEdit->text().length() - 1) != SEPARATOR)
        {
            mLineEdit->setText(mLineEdit->text() + SEPARATOR);
        }
    }
    else
    {
        mLineEdit->setText("0" + SEPARATOR + mLineEdit->text());
    }
}

void Calculator::BinaryOpClicked()
{
    if (mLineEdit->text().toStdString() == "")
    {
        return;
    }
    Equal();
    OneDot(false);
    if (OnlyDigits())
    {
        QString symbol = (qobject_cast<QPushButton*>(sender()))->text();
        if (mLineEdit->text().length())
        {
            if (symbol == "x\u207F")
            {
                mLineEdit->setText(mLineEdit->text() + ", x\u207F n = ");
            }
            else if (symbol == "\u207F\u221A")
            {
                mLineEdit->setText(mLineEdit->text() + ", \u207F\u221Ax n = ");
            }
            else
            {
                mLineEdit->setText(mLineEdit->text() + symbol);
            }
            SetDigits(false);
        }
        else if (symbol == "-")
        {
            mLineEdit->setText(symbol + mLineEdit->text());
        }
    }
}

double Calculator::fact(double p)
{
    if (p < 0)
    {
        return 0;
    }
    if (p == 0.0)
    {
        return 1;
    }
    else
    {
        return p * fact((int)floor(p) - 1);
    }
}


void Calculator::Clear()
{
    OneDot(false);
    mLineEdit->clear();
    SetDigits(true);
}

void Calculator::Equal()
{
    SetDigits(true);
    std::string expression = mLineEdit->text().toStdString();
    std::string search_slice = expression.substr(1);
    QString answer = "";

    if (search_slice.find('+') != std::string::npos)
    {
        std::string::size_type position;
        double leftNumber  = std::stod(expression, &position);
        double rightNumber = std::stod(expression.substr(position + 1));
        double answerNumber = leftNumber + rightNumber;
        answer = QString::number(answerNumber, 'f');

        while (*(answer.rbegin()) == '0')
        {
            answer.chop(1);
        }
        if (*(answer.rbegin()) == '.')
        {
            answer.chop(1);
        }

        mLineEdit->setText(answer);
    }
    else if (search_slice.find('-') != std::string::npos)
    {
        std::string::size_type position;
        double leftNumber  = std::stod(expression, &position);
        double rightNumber = std::stod(expression.substr(position + 1));
        double answerNumber = leftNumber - rightNumber;

        answer = QString::number(answerNumber, 'f');

        while (*(answer.rbegin()) == '0')
        {
            answer.chop(1);
        }
        if (*(answer.rbegin()) == '.')
        {
            answer.chop(1);
        }

        mLineEdit->setText(answer);
    }
    else if (search_slice.find("\u00D7") != std::string::npos)
    {
        std::string::size_type position;
        double leftNumber  = std::stod(expression, &position);
        double rightNumber = std::stod(expression.substr(position + 2));
        double answerNumber = leftNumber * rightNumber;

        answer = QString::number(answerNumber, 'f');

        while (*(answer.rbegin()) == '0')
        {
            answer.chop(1);
        }
        if (*(answer.rbegin()) == '.')
        {
            answer.chop(1);
        }

        mLineEdit->setText(answer);
    }
    else if (search_slice.find("\u00F7") != std::string::npos)
    {
        std::string::size_type position;
        double leftNumber  = std::stod(expression, &position);
        double rightNumber = std::stod(expression.substr(position + 2));
        double answerNumber = leftNumber / rightNumber;

        answer = QString::number(answerNumber, 'f');

        while (*(answer.rbegin()) == '0')
        {
            answer.chop(1);
        }
        if (*(answer.rbegin()) == '.')
        {
            answer.chop(1);
        }

        mLineEdit->setText(answer);
    }
    else if (search_slice.find(", x\u207F n = ") != std::string::npos)
    {
        std::string::size_type position;
        double leftNumber  = std::stod(expression, &position);
        double rightNumber = std::stod(mLineEdit->text().split(", x\u207F n = ")[1].toStdString());
        double answerNumber = std::pow(leftNumber, rightNumber);

        answer = QString::number(answerNumber, 'f');

        while (*(answer.rbegin()) == '0')
        {
            answer.chop(1);
        }
        if (*(answer.rbegin()) == '.')
        {
            answer.chop(1);
        }

        mLineEdit->setText(answer);
    }
    else if (search_slice.find(", \u207F\u221Ax n = ") != std::string::npos)
    {
        std::string::size_type position;
        double leftNumber  = std::stod(expression, &position);
        double rightNumber = std::stod(mLineEdit->text().split(", \u207F\u221Ax n = ")[1].toStdString());
        double answerNumber = std::pow(leftNumber, (1 / rightNumber));

        answer = QString::number(answerNumber, 'f');

        while (*(answer.rbegin()) == '0')
        {
            answer.chop(1);
        }
        if (*(answer.rbegin()) == '.')
        {
            answer.chop(1);
        }

        mLineEdit->setText(answer);
    }
}

bool Calculator::OnlyDigits()
{
    return mDigitsOonly;
}

